/* Global definitions for Reed-Solomon encoder/decoder
 * Phil Karn KA9Q, September 1996
 */
/* Set one of these to enable encoder/decoder debugging and error checking,
 * at the expense of speed */
/* #undef DEBUG 1 */
/* #undef DEBUG 2 */

/* To select the CCSDS standard (255,223) code, define CCSDS. This
 * implies standard values for MM, KK, B0 and PRIM.
 */
/* #undef CCSDS 1 */

#define MM 8

#define RSDSIZERS1 224
#define RSDSIZERS2 192
#define RSDSIZERS3 160
#define RSDSIZERS4 128


/*
 * Set KK to be the number of data symbols in each block, which must be
 * less than the block size. The code will then be able to correct up
 * to NN-KK erasures or (NN-KK)/2 errors, or combinations thereof with
 * each error counting as two erasures.
 */

/* Set B0 to the first root of the generator polynomial, in alpha form, and
 * set PRIM to the power of alpha used to generate the roots of the
 * generator polynomial. The generator polynomial will then be
 * @**PRIM*B0, @**PRIM*(B0+1), @**PRIM*(B0+2)...@**PRIM*(B0+NN-KK)
 * where "@" represents a lower case alpha.
 */
#define B0 1 /* First root of generator polynomial, alpha form */
#define PRIM 1 /* power of alpha used to generate roots of generator poly */

/* If you want to select your own field generator polynomial, you'll have
 * to edit that in rs.c.
 */
#define	NN ((1 << MM) - 1)
typedef unsigned char dtype;

/* Reed-Solomon encoding
 * data[] is the input block, parity symbols are placed in bb[]
 * bb[] may lie past the end of the data, e.g., for (255,223):
 *	encode_rs(&data[0],&data[223]);
 */
int encode_rs(dtype data[], dtype bb[]);

/* Reed-Solomon erasures-and-errors decoding
 * The received block goes into data[], and a list of zero-origin
 * erasure positions, if any, goes in eras_pos[] with a count in no_eras.
 *
 * The decoder corrects the symbols in place, if possible and returns
 * the number of corrected symbols. If the codeword is illegal or
 * uncorrectible, the data array is unchanged and -1 is returned
 */
int eras_dec_rs(dtype data[], int eras_pos[], int no_eras);
void init_rs(int kk);
