/*
*    File drmdefs.h
*
*    M.Bos - PA0MBO
*    Date Feb 21st 2009
*/

/*************************************************************************
*
*                           PA0MBO
*
*    COPYRIGHT (C)  2009  M.Bos 
*
*    This file is part of the distribution package RXAMADRM
*
*    This package is free software and you can redistribute is
*    and/or modify it under the terms of the GNU General Public License
*
*    More details can be found in the accompanying file COPYING
*************************************************************************/


#define DRMBUFSIZE 50000
#define Ts_A 320
#define Tu_A 288
#define Tg_A 32
#define Ts_B 320
#define Tu_B 256
#define Tg_B 64
#define Ts_C 240
#define Tu_C 160
#define Tg_C 64
#define Ts_D 200
#define Tu_D 112
#define Tg_D 88
#define RUN_STATE_POWER_ON -2
#define RUN_STATE_INIT -1
#define RUN_STATE_FIRST 0
#define RUN_STATE_NORMAL  1

