/* Automatically generated file with MATLAB */
/* File name: "ResampleFilter.m" */
/* Filter taps in time-domain */

#ifndef _RESAMPLEFILTER_H_
#define _RESAMPLEFILTER_H_

#define RES_FILT_NUM_TAPS_PER_PHASE  12
#define INTERP_DECIM_I_D             10

/* Filter for ratios close to 1 */
extern float fResTaps1To1[INTERP_DECIM_I_D][RES_FILT_NUM_TAPS_PER_PHASE];

#endif	/* _RESAMPLEFILTER_H_ */
