/* config.h.  Generated from config.h.in by configure.  */
/* config.h.in.  Generated from configure.in by autoheader.  */


/* Define to 1 if you have the <inttypes.h> header file. */
#define HAVE_INTTYPES_H 1



/* Define if you have libsndfile */
/* #undef HAVE_LIBSNDFILE */

/* Define to 1 if you have the `z' library (-lz). */
#define HAVE_LIBZ 0

/* Define to 1 if you have the <stdint.h> header file. */
#define HAVE_STDINT_H 1

/* Define to 1 if you have the <stdlib.h> header file. */
#define HAVE_STDLIB_H 1

/* Define to 1 if you have the <strings.h> header file. */
#define HAVE_STRINGS_H 1

/* Define to 1 if you have the <string.h> header file. */
#define HAVE_STRING_H 1

/* Define to 1 if you have the <sys/stat.h> header file. */
#define HAVE_SYS_STAT_H 1

/* Define to 1 if you have the <sys/types.h> header file. */
#define HAVE_SYS_TYPES_H 1

/* Define to 1 if you have the <unistd.h> header file. */
#define HAVE_UNISTD_H 1

/* Define to the sub-directory in which libtool stores uninstalled libraries.
   */
#define LT_OBJDIR ".libs/"

/* Name of package */
#define PACKAGE "drm"

/* Define to the address where bug reports for this package should be sent. */
#define PACKAGE_BUGREPORT ""

/* Define to the full name of this package. */
#define PACKAGE_NAME ""

/* Define to the full name and version of this package. */
#define PACKAGE_STRING ""

/* Define to the one symbol short name of this package. */
#define PACKAGE_TARNAME ""

/* Define to the home page for this package. */
#define PACKAGE_URL ""

/* Define to the version of this package. */
#define PACKAGE_VERSION ""

/* Define to 1 if you have the ANSI C header files. */
#define STDC_HEADERS 1

/* Define if you want to use the ALSA audio lib */
#define USE_ALSA 1

/* Define if you want to use the faac library */
/* #undef USE_FAAC_LIBRARY */

/* Define if you want to use the faad2 library */
/* #undef USE_FAAD2_LIBRARY */

/* Define if you want to use the jack audio lib */
/* #undef USE_JACK */

/* Define if you want to use the OSS audio lib */
/* #undef USE_OSS */

/* Define if you want to use the portaudio audio lib */
/* #undef USE_PORTAUDIO */

/* Define if you want to use the QT GUI */
/* #undef USE_QT_GUI */

/* Version number of package */
#define VERSION "1.12b"
