#ifndef LOGGINGPARAMS_H
#define LOGGINGPARAMS_H
#include <QString>

enum debugLevels {LOGGALLERY,LOGPARAM,LOGFFT,LOGWAVIO,LOGSYNTHES,LOGDISPATCH,LOGSOUND,
                  LOGRXMAIN,LOGTXMAIN,LOGRXFUNC,LOGTXFUNC,
                  LOGSYNCSTATE,LOGSYNCPROCESSSTATE,LOGSYNCQUALITY,LOGSYNCEXTRACT,LOGSYNCEVAL,LOGSYNCACCEPTED,LOGSYNCCOMP,LOGSYNCREJECTED,LOGSYNCMATCH,LOGSYNCLOST,LOGSLANT,
                  LOGMODES,LOGFTPFUNC,LOGFTPTHREAD,LOGQFTP,LOGIMAG,LOGSCOPE,LOGCAM,LOGEDIT,LOGRIGCTRL,
                  LOGDRMDEMOD,LOGDRMSRC,LOGDRMMOT,LOGDRMTX,LOGDRMTXMOT,LOGDRMTXCOD,LOGDRMTXAAA,LOGXML,
                  LOGFSKID,LOGVISCODE,LOGCALIB,LOGNOTIFY,LOGPERFORM,LOGALL=99};

#define NUMDEBUGLEVELS (LOGPERFORM+1)

extern QString levelStr[NUMDEBUGLEVELS];
#endif // LOGGINGPARAMS_H
